﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.ListGraphic
{

    /// <summary>
    /// （１）
    /// 「chrSymbol .. "_" .. gName」といった文字列連結式を、分解します。
    /// 
    /// （２）
    /// TODO: 「{"C:\aaa\bbb.lua*11"}chrSymbol .. "_" .. gName」は分解結果が複雑になってしまいます。
    /// 
    /// </summary>
    public abstract class Actions_ConcatTokenizer
    {

        public static void ShowDebug(List<String> tokenList)
        {
            StringBuilder sb = new StringBuilder();

            foreach (string token in tokenList)
            {
                sb.Append("[");
                sb.Append(token);
                sb.Append("]");
            }

            System.Console.WriteLine("★ShowDebug：　"+sb.ToString());
        }

        /// <summary>
        /// 「chrSymbol .. "_" .. gName」といった文字列連結式を想定しています。
        /// </summary>
        /// <param name="src"></param>
        /// <returns></returns>
        public static List<String> Tokenize(string src)
        {
            List<String> tokenList  = new List<string>();

            StringBuilder buffer = new StringBuilder();
            string bufferType = "";
            char preChar = ' ';

            //
            // 現版では、 \\\" の出現は考えていません。１つ前の円記号までです。
            //

            foreach (char ch in src)
            {
                if (bufferType == "LiteralString")
                {
                    if ('"' == ch && '\\' != preChar)
                    {
                        // リテラル文字列終了
                        bufferType = "";
                    }

                    buffer.Append(ch);
                }
                else if ('"' == ch && '\\' != preChar)
                {
                    if ("LiteralString" != bufferType)
                    {
                        if (0 < buffer.Length)
                        {
                            tokenList.Add(buffer.ToString());
                            buffer.Clear();
                        }

                        bufferType = "LiteralString";
                    }

                    buffer.Append(ch);
                }
                else if (Char.IsWhiteSpace(ch))
                {
                    if ("WhiteSpace" != bufferType)
                    {
                        if (0 < buffer.Length)
                        {
                            tokenList.Add(buffer.ToString());
                            buffer.Clear();
                        }

                        bufferType = "WhiteSpace";
                    }

                    buffer.Append(ch);
                }
                else if (Char.IsLetterOrDigit(ch))
                {
                    if ("LetterOrDigit" != bufferType)
                    {
                        if (0 < buffer.Length)
                        {
                            tokenList.Add(buffer.ToString());
                            buffer.Clear();
                        }

                        bufferType = "LetterOrDigit";
                    }

                    buffer.Append(ch);
                }
                else
                {
                    if ("Kigo" != bufferType)
                    {
                        if (0 < buffer.Length)
                        {
                            tokenList.Add(buffer.ToString());
                            buffer.Clear();
                        }

                        bufferType = "Kigo";
                    }
                    else
                    {
                        // 記号が連続しているとき
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        if ('{' == ch)
                        {
                            // 強制フラッシュ
                            tokenList.Add(buffer.ToString());
                            buffer.Clear();

                            // 後ろとつながらない
                            bufferType = "";
                        }
                        else if ('}' == ch)
                        {
                            // 強制フラッシュ
                            tokenList.Add(buffer.ToString());
                            buffer.Clear();

                            // 後ろとつながらない
                            bufferType = "";
                        }
                        else
                        {
                        }
                    }

                    buffer.Append(ch);
                }

                preChar = ch;
            }

            if (0 < buffer.Length)
            {
                tokenList.Add(buffer.ToString());
                buffer.Clear();
            }
            bufferType = "";


            return tokenList;
        }

    }
}
