﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Data;
using System.Text.RegularExpressions;
using System.IO;

namespace Xenon.ListGraphic
{
    /// <summary>
    /// loadGraphic 行一覧
    /// </summary>
    public class LoadGraphicTable
    {

        /// <summary>
        /// loadGraphic行一覧
        /// </summary>
        private DataTable Table
        {
            get;
            set;
        }

        public LoadGraphicTable()
        {
            this.Table = new DataTable();
            this.Table.Columns.Add("Program");
            this.Table.Columns.Add("Line", Type.GetType("System.Int32"));
            this.Table.Columns.Add("Name_Original");
            this.Table.Columns.Add("Name");
            this.Table.Columns.Add("Graphic");
            this.Table.Columns.Add("Comment");
            this.Table.Columns.Add("ErrorMessage");

            //関数
            this.Table.Columns.Add("InFunction1");
            this.Table.Columns.Add("Params1");

            //呼出し元関数
            this.Table.Columns.Add("Program2");
            this.Table.Columns.Add("Line2", Type.GetType("System.Int32"));
            this.Table.Columns.Add("Args2");
            this.Table.Columns.Add("Comment2");

            //論理削除フラグ
            this.Table.Columns.Add("Delete", Type.GetType("System.Boolean"));
        }

        public DataRow[] Select(String where, String orderBy)
        {
            return this.Table.Select(where, orderBy);
        }

        public DataRow NewRow()
        {
            return this.Table.NewRow();
        }

        public DataRowCollection Rows
        {
            get
            {
                return this.Table.Rows;
            }
        }

        private void Add(DataRow row)
        {
            this.Table.Rows.Add(row);
        }

        public void AddRow(string program, int line, string nameOriginal, string name, string graphic, string comment, string errorMessage, string inFunction1, string params1, string program2, int line2, string args2, string comment2)
        {
            DataRow row = this.NewRow();
            row["Program"] = program;
            row["Line"] = line;
            row["Name_Original"] = nameOriginal;
            row["Name"] = name;
            row["Graphic"] = graphic;
            row["Comment"] = comment;
            row["ErrorMessage"] = errorMessage;

            row["InFunction1"] = inFunction1;
            row["Params1"] = params1;

            row["Program2"] = program2;
            row["Line2"] = line2;
            row["Args2"] = args2;
            row["Comment2"] = comment2;
            row["Delete"] = false;
            this.Add(row);
        }

        public void ScanFileLines_1syume(string file, string[] lines)
        {
            //------------------------------
            //
            //  ３分割
            //
            //
            //  ★ = loadGraphic( ● ) -- ▲
            //  ↓
            // (1)★
            // (2)●
            // (3)▲
            //
            //
            //  想定するケース
            //
            //  (A) G.src["gauge"] = loadGraphic( "gfx/gauge.png" ) -- コメント
            //  (B) G.src [ chrSymbol .. "_g101_1"] = loadGraphic( "gfx/chr/" .. chrSymbol .. "/g101_1.png" ) -- コメント
            //
            //------------------------------
            Regex rZ = new Regex(
                @"\s*(.*)\s*" +                 //＜サンプル＞ G.src["gauge"]
                @"=\s*loadGraphic\s*\(\s*" +    //＜サンプル＞ = loadGraphic ( 
                @"(.*)" +                       //＜サンプル＞ "gfx/gauge.png"
                @"\)[^-]*" +                    //＜サンプル＞ )
                @"(?:--(.*))?\s*"               //＜サンプル＞ -- コメント
                , RegexOptions.Compiled | RegexOptions.Singleline
                );

            int lineCount = 1;
            foreach (string line in lines)
            {
                Match m = rZ.Match(line);
                if (m.Success)
                {
                    // loadGraphicの行でした。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                    //System.Console.WriteLine("★左辺＝["+m.Groups[1].Value+"]　loadGraphic実引数＝["+m.Groups[2].Value+"]　コメント＝["+m.Groups[3].Value+"]");

                    string sahenOriginal = m.Groups[1].Value; // 左辺     ＜サンプル＞   G.src[gName]
                    string args = m.Groups[2].Value; // 実引数     ＜サンプル＞  "chr .. "_" .. gName"

                    //------------------------------
                    //
                    //  左辺
                    //
                    //      G.src[chrSymbol .. "_g101_1"] 
                    //          ↓
                    //      G.src["kis801歩" .. "_g101_1"] 
                    //
                    //------------------------------
                    string sahenA;
                    string sahenB;
                    string sahenC;
                    string sahenExpanded;
                    if (Actions_BlacketOpener.OpenBlacket(sahenOriginal, out sahenA, out sahenB, out sahenC, file, lineCount))
                    {
                        //------------------------------
                        // 変数の頭に、マーカーを付けます。
                        //
                        //  ＜サンプル＞
                        //      chrSymbol .. "_"
                        //          ↓
                        //      {"c:\aaa\bbb.lua*12"}chrSymbol .. "_"
                        //
                        //------------------------------
                        string newSahenB;
                        Actions_AppendMarker.AppendMarker(sahenB, out newSahenB, file, lineCount);
                        sahenB = newSahenB;
                        //sahen = sahenA + newSahenB + sahenC;


                        // １周目
                        sahenExpanded = sahenA + sahenB + sahenC;
                        //System.Console.WriteLine("左辺　元＝[" + sahenOriginal + "]　新＝[" + sahenExpanded + "]　Ａ＝[" + sahenA + "]　★Ｂ＝[" + sahenB + "]　Ｃ＝[" + sahenC + "]");
                    }
                    else
                    {
                        sahenExpanded = sahenOriginal;
                    }



                    //------------------------------------------------------------
                    // loadGraphic の行
                    //------------------------------------------------------------
                    this.AddRow(
                        file,
                        lineCount,
                        sahenOriginal,
                        sahenExpanded,
                        args,
                        m.Groups[3].Value,
                        "",
                        "",
                        "",
                        "",
                        0,
                        "",
                        ""
                        );
                }

                lineCount++;
            }
        }



        /// <summary>
        /// report_graphic.csv
        /// </summary>
        /// <returns></returns>
        public string ReportCsv()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("No");
            sb.Append(",Program");
            sb.Append(",Line");
            sb.Append(",Name_Original");
            sb.Append(",Name");
            sb.Append(",Graphic");
            sb.Append(",Comment");
            sb.Append(",ErrorMessage");

            sb.Append(",InFunction1");
            sb.Append(",Params1");

            sb.Append(",Program2");
            sb.Append(",Line2");
            sb.Append(",Args2");
            sb.Append(",Comment2");

            sb.Append(",Delete");
            sb.AppendLine();

            int no = 1;
            foreach (DataRow row in this.Rows)
            {
                // 備考：　論理削除行も出力します

                sb.Append(Actions_Csv.EscapeCell(no.ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Program"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Line"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Name_Original"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Name"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Graphic"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Comment"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["ErrorMessage"].ToString()));

                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["InFunction1"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Params1"].ToString()));

                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Program2"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Line2"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Args2"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Comment2"].ToString()));

                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Delete"].ToString()));
                sb.AppendLine();

                no++;
            }

            return sb.ToString();
        }

        public string ReportHtml(string directoryPath, LocalTable localTable)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine(
@"<html>
<head>
    <title>レポート：　グラフィック</title>
<head>
<body>");


            int no = 1;
            foreach (DataRow row in this.Rows)
            {
                bool delete = (bool)row["Delete"];

                if (delete)
                {
                    // 論理削除されていました。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    goto gt_EndAdd;
                }

                sb.Append(@"<h1>No.");
                sb.Append(no.ToString());
                sb.Append(@"　");
                sb.Append(row["Comment"].ToString());
                string comment2 = row["Comment2"].ToString();
                if ("" != comment2)
                {
                    sb.Append(@"　");
                    sb.Append(comment2);
                }
                sb.Append(@"</h1>");
                sb.AppendLine();

                sb.Append(@"<div style=""margin-left:40px; margin-bottom:20px;"">");
                sb.AppendLine();
                {
                    // プログラム箇所
                    sb.Append(@"<a href=""");
                    sb.Append(row["Program"].ToString());
                    sb.Append(@""">");
                    sb.Append(row["Program"].ToString());
                    sb.Append(@"</a>");
                    sb.Append(@"　(");
                    sb.Append(row["Line"].ToString());
                    sb.Append(@")　");
                    sb.Append(@"<br/>");
                    sb.AppendLine();

                    // ハンドル名
                    string name = row["Name"].ToString();
                    //string mkFile;
                    //int mkLineCount;
                    //string name = Actions_Variablename.Demarking(row["Name"].ToString(), out mkFile, out mkLineCount);

                    //Actions_MarkerExpander.TryExpandMarker(Actions_ConcatTokenizer.Tokenize(name), localTable, out name);
                    sb.Append(name);

                    sb.Append(@"　=<br/>");
                    sb.AppendLine();

                    // 画像パス
                    string graphic = Actions_Text.TrimDoubleQuotation(row["Graphic"].ToString());
                    string graphicPath = directoryPath + Path.DirectorySeparatorChar + graphic;
                    //string graphicPath;
                    //{
                    //    StringBuilder sb2 = new StringBuilder();
                    //    sb2.Append(directoryPath);
                    //    sb2.Append(Path.DirectorySeparatorChar);
                    //    sb2.Append(graphic);

                    //    List<string> tokenList = new List<string>();
                    //    tokenList.Add(sb2.ToString());
                    //    Actions_MarkerExpander.TryExpandMarker(tokenList, localTable, out graphicPath);
                    //}

                    sb.Append(@"　　　　<a href=""");
                    sb.Append(graphicPath);
                    sb.Append(@""">");

                    sb.Append(graphicPath);

                    sb.Append(@"</a><br/>");
                    sb.AppendLine();

                    // 画像
                    sb.Append(@"　<img src=""");
                    sb.Append(directoryPath);
                    sb.Append(Path.DirectorySeparatorChar);
                    sb.Append(graphic);
                    sb.Append(@""" alt=""");
                    sb.Append(row["Name"].ToString());
                    sb.Append(@"""><br/>");
                    sb.AppendLine();

                }
                sb.Append(@"</div>");
                sb.AppendLine();
                sb.AppendLine();

                no++;

            gt_EndAdd:
                ;
            }



            sb.AppendLine(
@"</body>
</html>");


            return sb.ToString();
        }

        public void Merge(LoadGraphicTable table2)
        {
            this.Table.Merge(table2.Table);

            DataRow[] dataRows = this.Table.Select(null, "Program,Line,Program2,Line2");

            DataTable table3 = this.Table.Clone();
            foreach(DataRow row in dataRows)
            {
                table3.ImportRow(row);
            }

            this.Table = table3;
        }

    }
}
