﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Data;
using System.Text.RegularExpressions;

namespace Xenon.ListGraphic
{

    /// <summary>
    /// local の行一覧
    /// </summary>
    public class LocalTable
    {

        /// <summary>
        /// local の行
        /// </summary>
        private DataTable Table
        {
            get;
            set;
        }


        public LocalTable()
        {
            this.Table = new DataTable();
            this.Table.Columns.Add("Program");
            this.Table.Columns.Add("Line", Type.GetType("System.Int32"));
            this.Table.Columns.Add("Local");
            this.Table.Columns.Add("Name");
            this.Table.Columns.Add("Value");
            this.Table.Columns.Add("Comment");
        }


        public DataRow[] Select(String where, String orderBy)
        {
            return this.Table.Select(where, orderBy);
        }

        public DataRow NewRow()
        {
            return this.Table.NewRow();
        }

        public DataRowCollection Rows
        {
            get
            {
                return this.Table.Rows;
            }
        }

        public void Add(DataRow row)
        {
            this.Table.Rows.Add(row);
        }


        public void ScanFileLines_1syume(string file, string[] lines)
        {
            //------------------------------
            //
            // 変換   ＜サンプル＞
            //
            // local chrSymbol = "kis801歩"
            // ↓
            // local
            // chrSymbol
            // kis801歩
            //
            //------------------------------
            Regex rA = new Regex(
                @"^\s*"+                //隙間
                @"(local)?\s*" +        //＜サンプル＞ local
                @"([^~=\s]*)\s*" +      //＜サンプル＞ chrSymbol
                @"(=|~=)\s*"+           //＜サンプル＞ = または ~=
                @"("".*"")\s*" +         //＜サンプル＞ "kis801歩"
                @"(?:[-][-](.*))?\s*"   //＜サンプル＞ -- コメント
                , RegexOptions.Compiled | RegexOptions.Singleline);

            //------------------------------
            //
            // 変換   ＜サンプル＞
            //
            // local gName1 = chrSymbol .. "_" .. graName
            // ↓
            // local
            // gName1
            // chrSymbol .. "_" .. graName
            //
            //------------------------------
            Regex rB = new Regex(
                @"^\s*" +                //隙間
                @"(local)?\s*" +        //＜サンプル＞ local
                @"([^~=\s]*)\s*" +      //＜サンプル＞ chrSymbol
                @"(=|~=)\s*" +           //＜サンプル＞ = または ~=
                @"((?:[^\s]+\s*\.\.\s*)?"".*""\s*(?:\.\.\s*[^\s])?)" +         //＜サンプル＞ "kis801歩"
                @"(?:[-][-](.*))?\s*"   //＜サンプル＞ -- コメント
                , RegexOptions.Compiled | RegexOptions.Singleline);
            //Regex rB = new Regex(
            //    @"^\s*" +                //隙間
            //    @"(local)?\s*" +        //＜サンプル＞ local
            //    @"([^~=\s]*)\s*" +      //＜サンプル＞ chrSymbol
            //    @"(=|~=)\s*" +           //＜サンプル＞ = または ~=
            //    @"((?:[^\s]+\s*\.\.\s*)?"".*""\s*(?:\.\.\s*[^\s])?)" +         //＜サンプル＞ "kis801歩"
            //    @"(?:[-][-](.*))?\s*"   //＜サンプル＞ -- コメント
            //    , RegexOptions.Compiled | RegexOptions.Singleline);


            int lineCount = 1;
            foreach (string line in lines)
            {
                this.ScanFileLineA(file, lineCount, rA, line);
                this.ScanFileLineA(file, lineCount, rB, line);

                lineCount++;
            }
        }


        private void ScanFileLineA(string file, int lineCount, Regex r, string line)
        {
            Match m = r.Match(line);
            if (m.Success)
            {
                if ("~=" == m.Groups[3].Value)
                {
                    // これは代入ではありません。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    goto gt_EndParse;
                }

                //------------------------------------------------------------
                // 代入文
                //------------------------------------------------------------
                DataRow row = this.NewRow();
                row["Program"] = file;
                row["Line"] = lineCount;
                row["Local"] = m.Groups[1]; //"local";
                row["Name"] = m.Groups[2];
                row["Value"] = m.Groups[4].Value;
                row["Comment"] = m.Groups[5];
                this.Add(row);
                goto gt_EndParse;
            }

        gt_EndParse:
            ;
        }


        /// <summary>
        /// report_local.csv
        /// </summary>
        /// <returns></returns>
        public string ReportCsv()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("No");
            sb.Append(",Program");
            sb.Append(",Line");
            sb.Append(",Local");
            sb.Append(",Name");
            sb.Append(",Value");
            sb.Append(",Comment");
            sb.AppendLine();

            int no = 1;
            foreach (DataRow row in this.Rows)
            {
                sb.Append(Actions_Csv.EscapeCell(no.ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Program"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Line"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Local"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Name"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Value"].ToString()));
                sb.Append(",");
                sb.Append(Actions_Csv.EscapeCell(row["Comment"].ToString()));
                sb.AppendLine();

                no++;
            }

            return sb.ToString();
        }

        /// <summary>
        /// ソースを遡ってlocal宣言を調べます。
        /// </summary>
        /// <param name="program1"></param>
        /// <param name="name2"></param>
        /// <param name="line1"></param>
        /// <returns></returns>
        public DataRow FindLocalDecralation(string program1, string name2, int line1)
        {
            DataRow localRow = null;

            StringBuilder sb = new StringBuilder();

            sb.Append("Program like '");
            sb.Append(program1);
            sb.Append("' and ");

            sb.Append("Name like '");
            sb.Append(name2);
            sb.Append("' and ");

            sb.Append("Line<");
            sb.Append(line1);

            DataRow[] localRows = this.Select(sb.ToString(), "Line");

            if (0 < localRows.Length)
            {
                localRow = localRows[localRows.Length - 1];
            }

            return localRow;
        }

    }

}
