﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO;
using System.Text.RegularExpressions;


namespace Xenon.ListGraphic
{
    public partial class UcJikkou : UserControl
    {

        #region プロパティー類
        public string ErrorMessage
        {
            get;
            set;
        }

        private bool completed;
        /// <summary>
        /// 処理が正常終了していれば真。
        /// </summary>
        public bool Completed
        {
            get
            {
                return this.completed;
            }
        }

        public string DirectoryPath
        {
            get;
            set;
        }

        /// <summary>
        /// .lua ファイル一覧
        /// </summary>
        private List<string> Report_LuaFiles
        {
            get;
            set;
        }

        /// <summary>
        /// loadGraphic行一覧
        /// </summary>
        public LoadGraphicTable Report_LoadGraphic
        {
            get;
            set;
        }

        /// <summary>
        /// loadGraphic行一覧  ＜追加用＞
        /// </summary>
        private LoadGraphicTable Report_LoadGraphic_Append
        {
            get;
            set;
        }

        /// <summary>
        /// local行一覧
        /// </summary>
        public LocalTable Report_Local
        {
            get;
            set;
        }

        /// <summary>
        /// function行一覧
        /// </summary>
        public FunctionTable Report_Function
        {
            get;
            set;
        }

        /// <summary>
        /// 関数呼出し行一覧
        /// </summary>
        public FunctioncallTable Report_Functioncall
        {
            get;
            set;
        }

        private int ProgressCurrent
        {
            get;
            set;
        }

        private int ProgressTotal
        {
            get;
            set;
        }

        private int ProgressPercent
        {
            get
            {
                int percent = 0;

                if (0 != this.ProgressTotal)
                {
                    percent = (int)((double)this.ProgressCurrent * 100.0d / (double)this.ProgressTotal);
                }

                return percent;
            }
        }
        #endregion



        public UcJikkou()
        {
            InitializeComponent();
        }

        private void UcJikkou_Load(object sender, EventArgs e)
        {
            this.Report_LuaFiles = new List<string>();
            this.ErrorMessage = "";

            // BackgroundWorkerのProgressChangedイベントが発生するようにします。
            this.backgroundWorker1.WorkerReportsProgress = true;

            //キャンセルできるようにします。
            this.backgroundWorker1.WorkerSupportsCancellation = true;

            //------------------------------------------------------------
            // スタート
            //------------------------------------------------------------
            this.backgroundWorker1.RunWorkerAsync();

        }

        /// <summary>
        /// 実行中
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {

            //------------------------------
            // クリアー
            //------------------------------
            this.ErrorMessage = "";
            this.completed = false;
            this.ProgressCurrent = 0;
            this.ProgressTotal = 0;

            this.Report_LuaFiles.Clear();

            this.Report_LoadGraphic = new LoadGraphicTable();
            this.Report_LoadGraphic_Append = new LoadGraphicTable();
            this.Report_Local = new LocalTable();
            this.Report_Function = new FunctionTable();
            this.Report_Functioncall = new FunctioncallTable();

            Directory.CreateDirectory(Actions_File.Path_ReportDirectory);

            //------------------------------
            // ファイルの使用可能チェック
            //------------------------------
            string errorMessage;
            if (!Actions_File.CanWrite(Actions_File.Path_ReportLoadgraphicCsv, out errorMessage))
            {
                this.ErrorMessage = errorMessage;
                goto gt_EndMethod;
            }

            if (!Actions_File.CanWrite(Actions_File.Path_ReportLocalCsv, out errorMessage))
            {
                this.ErrorMessage = errorMessage;
                goto gt_EndMethod;
            }

            if (!Actions_File.CanWrite(Actions_File.Path_ReportFunctionCsv, out errorMessage))
            {
                this.ErrorMessage = errorMessage;
                goto gt_EndMethod;
            }

            if (!Actions_File.CanWrite(Actions_File.Path_ReportFunctioncallCsv, out errorMessage))
            {
                this.ErrorMessage = errorMessage;
                goto gt_EndMethod;
            }

            if (!Actions_File.CanWrite(Actions_File.Path_ReportGraphicHtml, out errorMessage))
            {
                this.ErrorMessage = errorMessage;
                goto gt_EndMethod;
            }


            //------------------------------
            // 走査：  全ファイル
            //------------------------------
            this.Scan1(this.DirectoryPath);

            this.ProgressTotal = this.Report_LuaFiles.Count;

            //------------------------------
            // 開く：　全ファイル
            //------------------------------
            foreach (string file in this.Report_LuaFiles)
            {
                //キャンセルされたか調べる
                if (this.backgroundWorker1.CancellationPending)
                {
                    //キャンセルされたとき
                    e.Cancel = true;
                    goto gt_EndMethod;
                }

                Application.DoEvents();// 時間のかかる処理がある場合は、これを挟みます。

                this.Scan2InFile(file);

                this.ProgressCurrent++;
                // ProgressChangedイベントハンドラを呼び出し、コントロールの表示を変更します
                StringBuilder sb = new StringBuilder();
                sb.Append("(");
                sb.Append(this.ProgressCurrent);
                sb.Append("/");
                sb.Append(this.ProgressTotal);
                sb.Append(") ");
                sb.Append(file);
                this.backgroundWorker1.ReportProgress(this.ProgressPercent, sb.ToString());
            }

            //------------------------------
            // ローカル変数の解読
            //------------------------------
            this.Scan3Expand();

            //------------------------------
            // マージ
            //------------------------------
            this.Report_LoadGraphic.Merge(this.Report_LoadGraphic_Append);

            this.completed = true;

        gt_EndMethod:
            ;
        }

        private void Scan1( string directoryPath )
        {
            string[] entries = Directory.GetFileSystemEntries(directoryPath);

            foreach (string entry in entries)
            {
                if (FileAttributes.Directory == (File.GetAttributes(entry) & FileAttributes.Directory))
                {
                    // ディレクトリーなら
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    this.Scan1(entry);
                }
                else
                {
                    // ファイルなら
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                    if (".lua" == Path.GetExtension(entry).Trim().ToLower())
                    {
                        // .lua ファイルなら
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        this.Report_LuaFiles.Add(entry);
                    }
                }
            }
        }

        /// <summary>
        /// ファイルの中の loadGraphics
        /// </summary>
        /// <param name="file"></param>
        private void Scan2InFile(string file)
        {
            string[] lines = File.ReadAllLines(file, Encoding.GetEncoding("Shift-JIS"));

            //------------------------------------------------------------
            // loadGraphic の行   ＜１周目＞
            //------------------------------------------------------------
            this.Report_LoadGraphic.ScanFileLines_1syume(file, lines);

            //------------------------------------------------------------
            // local の行         ＜１周目＞
            //------------------------------------------------------------
            this.Report_Local.ScanFileLines_1syume(file, lines);

            //------------------------------------------------------------
            // function の行      ＜１周目＞
            //------------------------------------------------------------
            this.Report_Function.ScanFileLines_1syume(file, lines);

            //------------------------------------------------------------
            // 関数呼出し行       ＜１周目＞
            //------------------------------------------------------------
            this.Report_Functioncall.ScanFileLines_1syume(file, lines);
        }


        /// <summary>
        /// ローカル変数の解読
        /// </summary>
        private void Scan3Expand()
        {

            //------------------------------
            //
            // 分解Ａ   ＜サンプル＞
            //
            // src[gName]
            // ↓
            // (1)src[
            // (2)gName
            // (3)]
            //
            //------------------------------
            Regex rA = new Regex(
                                    @"([^[]*\[)"    + //＜サンプル＞　src[
                                    @"\s*"          + //隙間
                                    @"([^\s""\]]*)" + //＜サンプル＞ gName
                                    @"\s*"          + //隙間
                                    @"(\])"           //＜サンプル＞ ]
                                    //@"(\].*)"      //＜サンプル＞ ]
                                    , RegexOptions.Compiled | RegexOptions.Singleline);

            /*
            //------------------------------
            //
            // 変換Ｃ   ＜サンプル＞
            //
            // src[{C:\aaa\bbb.lua*63}gName .. "/g101_1.png"]
            // ↓
            // (1)src[
            // (2){C:\aaa\bbb.lua*63}
            // (3)gName
            // (4)/g101_1.png"]
            //
            //------------------------------
            Regex rC = new Regex(
                                    @"([^[]*\[)\s*" +   //＜サンプル＞ src[
                                    @"(\{[^\*]+\*\d+\})?\s*" +//＜サンプル＞ {C:\aaa\bbb.lua*63}
                                    @"([^\s\.""]*)\s*" +   //＜サンプル＞ gName
                                    @"[.][.]\s*""(.*)"  //＜サンプル＞ .. "/g101_1.png"]
                                    , RegexOptions.Compiled | RegexOptions.Singleline);
             */

            //------------------------------
            //
            // 変換   ＜サンプル＞
            //
            // （Ｄ）
            // G.src[chrSymbol .. "_" .. g]
            // ↓
            // G.src[
            // chrSymbol
            // _
            // g]
            // 
            //------------------------------
            //Regex rD = new Regex(
            //            @"([^[]*[[])\s*" +   //＜サンプル＞ src[gName
            //            @"([^\s\.""]*)\s*" +   //＜サンプル＞ gName
            //            @"[.][.]\s*""(.*)"  //＜サンプル＞ .. "/g101_1.png"]
            //            , RegexOptions.Compiled | RegexOptions.Singleline);


            //------------------------------
            //
            // 分解Ｚ１   ＜サンプル＞
            //
            // gfx/chr/" .. chrSymbol .. "/g101_1.png
            // ↓
            // gfx/chr/
            // chrSymbol
            // /g101_1.png
            //
            //------------------------------
            Regex rZ1 = new Regex(
                                    @"(.*)""\s*[.][.]\s*" +  //＜サンプル＞ gfx/chr/" .. 
                                    @"([^\s\.""]*)\s*" +       //＜サンプル＞ chrSymbol
                                    @"[.][.]\s*""(.*)"      //＜サンプル＞ .. "/g101_1.png
                                    , RegexOptions.Compiled | RegexOptions.Singleline);



            foreach (DataRow loadGraphicRow in this.Report_LoadGraphic.Rows)
            {
                string program1 = (string)loadGraphicRow["Program"];
                int line1 = (int)loadGraphicRow["Line"];


                //------------------------------------------------------------
                // 変数展開：　画像ハンドル
                //------------------------------------------------------------
                this.Scan3TranslateNameA(loadGraphicRow, "Name", rA, program1, line1);

                //this.Scan3TranslateName(loadGraphicRow, "Name", rB, program1, line1, "", "");

                //this.Scan3TranslateNameC(loadGraphicRow, "Name", rC, program1, line1);


                //------------------------------------------------------------
                // 変数展開：　画像ファイルパス
                //------------------------------------------------------------
                this.Scan3TranslateGraphic(loadGraphicRow, "Graphic", rZ1, program1, line1, "", "");

            }
        }

        /// <summary>
        ///------------------------------
        ///
        /// 分解Ａ   ＜サンプル＞
        ///
        /// src[gName]
        /// ↓
        /// (1)src[
        /// (2)gName
        /// (3)]
        ///
        ///------------------------------
        /// </summary>
        /// <param name="loadGraphicRow"></param>
        /// <param name="changeColumnName"></param>
        /// <param name="r"></param>
        /// <param name="program1"></param>
        /// <param name="lineCount1"></param>
        /// <param name="padding1"></param>
        /// <param name="padding2"></param>
        private void Scan3TranslateNameA(DataRow loadGraphicRow, string changeColumnName, Regex r, string program1, int lineCount1)
        {
            string target = (string)loadGraphicRow[changeColumnName];

            string result;
            Actions_MarkerExpander.TryExpandMarker( Actions_ConcatTokenizer.Tokenize(target), this.Report_Local, out result);
            loadGraphicRow[changeColumnName] = result;

            /*
            Match m = r.Match(target);
            while (m.Success)
            {
                // 画像ハンドル（変数名）の中に、変数がありました。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


                string expression = m.Groups[2].Value;



                //------------------------------
                // gName
                // ↓
                // "銀"
                //------------------------------
                string graphicName;
                string errorMessage;
                if (!Actions_Variablename.ExpandVariable(program1, lineCount1, expression, out graphicName, this.Report_Local, out errorMessage))
                {
                    loadGraphicRow["InFunction1"] = errorMessage;
                    goto gt_EndMethod;
                }


                //------------------------------
                // 文字列の再構成
                //------------------------------
                {
                    StringBuilder sb = new StringBuilder();

                    sb.Append(m.Groups[1]);
                    sb.Append(graphicName);
                    sb.Append(m.Groups[3]);

                    loadGraphicRow[changeColumnName] = sb.ToString();
                }

                //------------------------------
                // 再検索
                //------------------------------
                m = r.Match((string)loadGraphicRow[changeColumnName]);
            }
             */

        gt_EndMethod:
            ;
        }

        /*
        /// <summary>
        ///------------------------------
        ///
        /// 変換Ｃ   ＜サンプル＞
        ///
        /// src[{C:\aaa\bbb.lua*63}gName .. "/g101_1.png"]
        /// ↓
        /// (1)src[
        /// (2){C:\aaa\bbb.lua*63}
        /// (3)gName
        /// (4)/g101_1.png"]
        ///
        ///------------------------------
        /// </summary>
        /// <param name="loadGraphicRow"></param>
        /// <param name="changeColumnName"></param>
        /// <param name="r"></param>
        /// <param name="program1"></param>
        /// <param name="lineCount1"></param>
        private void Scan3TranslateNameC(DataRow loadGraphicRow, string changeColumnName, Regex r, string program1, int lineCount1)
        {
            string target = (string)loadGraphicRow[changeColumnName];

            Match m = r.Match(target);
            while (m.Success)
            {
                // 画像ハンドル（変数名）の中に、変数がありました。
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                string sahen = m.Groups[2].Value;

                string mkFile;
                int mkLineCount;
                string mkName = Actions_Variablename.Demarking(sahen, out mkFile, out mkLineCount);

                if ("" == mkFile && 0 == mkLineCount)
                {
                    mkFile = program1;
                    mkLineCount = lineCount1;
                }

                string variableName = m.Groups[3].Value;
                //System.Console.WriteLine("★variableName=[" + variableName + "]");

                //------------------------------
                // 検索：ローカル変数
                //------------------------------
                //DataRow localRow = this.Report_Local.FindLocalDecralation(program1, variableName, line1);
                DataRow localRow = this.Report_Local.FindLocalDecralation(mkFile, variableName, mkLineCount);

                if (null == localRow)
                {
                    // ローカル変数が使われているが、関数内には　該当するローカル変数はなかった。

                    loadGraphicRow["InFunction1"] = "Error(Name):　変数「" + variableName + "」が使われていましたが、該当するローカル変数はありませんでした。["+mkFile+"]["+mkLineCount+"]";
                    goto gt_EndMethod;
                }

                //------------------------------
                // 文字列の再構成
                //------------------------------
                {
                    StringBuilder sb = new StringBuilder();

                    sb.Append(m.Groups[1]);
                    sb.Append(@"""");
                    sb.Append( Actions_Text.TrimDoubleQuotation( (string)localRow["Value"] ) );
                    sb.Append(m.Groups[4]);

                    loadGraphicRow[changeColumnName] = sb.ToString();
                }

                //------------------------------
                // 再検索
                //------------------------------
                m = r.Match((string)loadGraphicRow[changeColumnName]);
            }

        gt_EndMethod:
            ;
        }
         */

        /// <summary>
        /// 
        /// </summary>
        /// <param name="rowA">loadGraphic関数の行</param>
        /// <param name="changeColumnName"></param>
        /// <param name="rA"></param>
        /// <param name="program1"></param>
        /// <param name="line1"></param>
        /// <param name="padding1"></param>
        /// <param name="padding2"></param>
        private void Scan3TranslateGraphic(DataRow rowA, string changeColumnName, Regex rA, string program1, int line1, string padding1, string padding2)
        {
            Match m = rA.Match((string)rowA[changeColumnName]);
            while (m.Success)
            {
                //------------------------------
                // ローカル変数の検索
                //------------------------------
                DataRow rowB = this.Report_Local.FindLocalDecralation(program1, m.Groups[2].Value, line1);

                if (null == rowB)
                {
                    // 変数はあったが、該当するローカル変数はなかった。

                    DataRow rowC = this.Report_Function.FindFunctionSignature(program1, line1);

                    if (null == rowC)
                    {
                        rowA["InFunction1"] = "Error(Graphic):　変数「" + m.Groups[2].Value + "」が使われていましたが、該当するローカル変数はありませんでした。";
                        goto gt_EndMethod;
                    }

                    // loadGraphic が書かれている関数の名前は分かりました。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                    string functionName = (string)rowC["Name"];
                    rowA["InFunction1"] = functionName;

                    DataRow[] rowsD = this.Report_Functioncall.FindAllFunctioncall(functionName);

                    if (rowsD.Length <= 0)
                    {
                        rowA["ErrorMessage"] = "Error(Graphic):　loadGraphicは、関数「" + functionName + "」の中に書かれていましたが、この関数の呼出しはありませんでした。";
                        goto gt_EndMethod;
                    }

                    //------------------------------
                    // 行削除
                    //------------------------------
                    //
                    //      関数呼出しごとにレコードを作成するので、関数定義の行は消します。
                    //
                    rowA["Delete"] = true;


                    // 関数呼出し箇所が分かりました。
                    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                    foreach (DataRow rowD in rowsD)
                    {
                        // 仮引数
                        string cParams = (string)rowC["Params"];
                        List<string> paramsList = Actions_Csv.UnescapeLineToFieldList(cParams, ',');

                        // 実引数を取り出します。
                        string dArgs = (string)rowD["Args"];
                        List<string> argsList = Actions_Csv.UnescapeLineToFieldList(dArgs, ',');

                        //System.Console.WriteLine("★fnParams=[" + cParams + "]　args=[" + dArgs + "]");

                        Dictionary<string, string> parameters = new Dictionary<string, string>();
                        if(paramsList.Count!=argsList.Count)
                        {
                            rowA["ErrorMessage"] = "Error:　仮引数「" + paramsList.Count + "」個、実引数「" + argsList.Count + "」個で数が合いませんでした。";
                            goto gt_EndMethod;
                        }

                        for (int i = 0; i < paramsList.Count; i++ )
                        {
                            parameters.Add(paramsList[i], argsList[i]);
                        }

                        //------------------------------
                        // graphics を置換したい
                        //------------------------------
                        string graphic = (string)rowA["Graphic"];

                        foreach(KeyValuePair<string,string> pair in parameters)
                        {
                            // " .. aaaa .. " 形式
                            Regex r1 = new Regex(
                                                    @"""\s*" +      //＜サンプル＞ "
                                                    @"\.\.\s*" +    //＜サンプル＞ ..
                                                    pair.Key +      //＜サンプル＞ aaaa
                                                    @"\s*\.\.\s*"+  //＜サンプル＞ ..
                                                    @"""\s*"        //＜サンプル＞ "
                                                    , RegexOptions.Compiled | RegexOptions.Singleline);

                            string value = pair.Value.Trim();
                            if (2 <= value.Length && value.StartsWith(@"""") && value.EndsWith(@""""))
                            {
                                // リテラル文字列でした。＜サンプル＞ "aaaa"
                                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                                value = value.Substring(1, value.Length - 2);
                            }
                            else
                            {
                                // 変数名かもしれません。
                                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                                DataRow localRow = this.Report_Local.FindLocalDecralation((string)rowD["Program"], value, (int)rowD["Line"]);
                                if (null == localRow)
                                {
                                    rowA["ErrorMessage"] = "Error:　関数呼出しの実引数に、ローカル変数「" + value + "」が指定されていましたが、そこの関数内に 「local "+value+"」 宣言が見つからなかったので検索を中断しました。";
                                    goto gt_EndMethod;
                                }

                                // ローカル変数を展開。
                                value = Actions_Text.TrimDoubleQuotation((string)localRow["Value"]);
                            }

                            graphic = r1.Replace(graphic, value);
                        }



                        this.Report_LoadGraphic_Append.AddRow(
                            (string)rowA["Program"],
                            (int)rowA["Line"],
                            (string)rowA["Name_Original"],
                            (string)rowA["Name"],
                            graphic,
                            (string)rowA["Comment"],
                            (string)rowA["ErrorMessage"],
                            //
                            (string)rowA["InFunction1"],
                            cParams,
                            //
                            (string)rowD["Program"],
                            (int)rowD["Line"],
                            dArgs,
                            (string)rowD["Comment"]
                            );
                    }

                    goto gt_EndMethod;
                }

                //------------------------------
                // 文字列の再構成
                //------------------------------
                //
                //  Name列や、Graphic列の変数を展開します。
                //
                {
                    StringBuilder sb = new StringBuilder();

                    sb.Append(m.Groups[1]);
                    sb.Append(padding1);
                    sb.Append(Actions_Text.TrimDoubleQuotation((string)rowB["Value"]));
                    sb.Append(padding2);
                    sb.Append(m.Groups[3]);

                    rowA[changeColumnName] = sb.ToString();
                }

                //------------------------------
                // 再検索
                //------------------------------
                m = rA.Match((string)rowA[changeColumnName]);
            }

        gt_EndMethod:
            ;
        }


        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                //------------------------------------------------------------
                // エラー
                //------------------------------------------------------------
                this.lblStatus.Text = "エラー:　" + e.Error.Message;
            }
            else if (e.Cancelled || ""!=this.ErrorMessage)
            {
                //------------------------------------------------------------
                // キャンセル、または エラー
                //------------------------------------------------------------
                this.ParentForm.DialogResult = DialogResult.Cancel;
                this.ParentForm.Close();
            }
            else
            {
                //------------------------------------------------------------
                // ダイアログを閉じます
                //------------------------------------------------------------
                this.ParentForm.DialogResult = DialogResult.OK;
                this.ParentForm.Close();
            }
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            //------------------------------
            // プログレス・バー
            //------------------------------
            this.progressBar1.Value = e.ProgressPercentage;

            //Label1のテキストを変更する
            this.lblStatus.Text = (string)e.UserState;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------
            // キャンセル
            //------------------------------------------------------------
            this.backgroundWorker1.CancelAsync();
        }
    }
}
